@@ -223,7 +223,7 @@ class Node:
             if field not in kw:
                 kw[field] = getattr(self, field)
         result = self.__class__(**kw)
-        result._metadata._fields.update(self._metadata._fields)
+        result._metadata.update(self._metadata)
         return result
 
 
@@ -237,9 +237,15 @@ class _Metadata:
     def __setattr__(self, name, value):
         self._fields[name] = value
 
+    def __len__(self):
+        return len(self._fields)
+
     def copy(self):
         return _Metadata(**self._fields)
 
+    def update(self, other):
+        self._fields.update(other._fields)
+
 
 class Rule:
     def __init__(self, name, parse, definition):
@@ -388,6 +394,56 @@ def visit(node):
                 stack.extend(getattr(node, x) for x in node._fields)
 
 
+_Traversing = _nt('_Traversing', 'parent, field, child, is_finished')
+
+
+def traverse(node):
+    visited = set()
+    stack = [_Traversing(parent=None, field=None, child=node, is_finished=False)]
+    while stack:
+        traversing = stack.pop()
+
+        if traversing.is_finished:
+            yield traversing
+            continue
+
+        child = traversing.child
+        child_id = id(child)
+
+        if child_id in visited:
+            continue
+
+        visited.add(child_id)
+        stack.append(traversing._replace(is_finished=True))
+        yield traversing
+
+        def extend(items):
+            stack.extend(reversed(list(items)))
+
+        if isinstance(child, (list, tuple)):
+            extend(
+                _Traversing(parent=child, field=i, child=x, is_finished=False)
+                for i, x in enumerate(child)
+            )
+
+        elif isinstance(child, dict):
+            extend(
+                _Traversing(parent=child, field=k, child=v, is_finished=False)
+                for k, v in child.items()
+            )
+
+        elif isinstance(child, Node) and hasattr(child, '_fields'):
+            extend(
+                _Traversing(
+                    parent=child,
+                    field=x,
+                    child=getattr(child, x),
+                    is_finished=False,
+                )
+                for x in child._fields
+            )
+
+
 def transform(node, *callbacks):
     if not callbacks:
         return node
@@ -414,8 +470,10 @@ def _transform(node, callback):
     for field in node._fields:
         was = getattr(node, field)
         now = _transform(was, callback)
-        if was is not now:
+        if now is not was:
             updates[field] = now
+            if isinstance(was, Node) and isinstance(now, Node) and not now._metadata:
+                now._metadata.update(was._metadata)
 
     if updates:
         node = node._replace(**updates)
